/* 
    ogive_and_ogee.scad - 
    parametric ogive and ogee modules for OpenSCAD
    Stewart Russell - scruss - 2018

Copyright (c) 2018 Stewart Russell (scruss)

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/

module ogive(width=10, height=10, steps=16) {
/*

 returns half an ogive polygon (likely 'secant ogive') suitable for use with rotate_extrude() and mirror().
    
 Parameters:
    
    width - width of half profile. Default is 10.
    
    height - height of half profile. Default is 10.
    
    steps - number of steps per face. Use a multiple of 4 for tidy results (for some unknown reason). Default is 16. You can go as low as 4.
    
 Example: generate an equilateral arch profile of width 20 and height 17.32:
    
    union(){
        ogive(10 * cos(60), 10 * sin(60));
        mirror()ogive(10 * cos(60), 10 * sin(60));
    }
    
 Formula: Elevation z is given by
    
    z = sqrt((3 - 2 * t - t^2) / 3)
    
 where t = 0 .. 1, returning z = 1 .. 0.

 */
    pts = [ for (t = [0 : 1 / steps : 1]) [ t, sqrt((3 - 2 * t-pow(t, 2)) / 3) ] ];
    scale([width, height])polygon(concat([ [0, 0] ], pts));
}

module ogee(width=10, height=10, steps=16) {
/*
    returns half an ogee polygon suitable for use with rotate_extrude() and mirror().
    
 Parameters:
    
    width - width of half profile. Default is 10.
    
    height - height of half profile. Default is 10.
    
    steps - number of steps per face. Use a multiple of 4 for tidy results (for some unknown reason). Default is 16. You can go as low as 4, but it looks terrible.
    
 Example: generate an ogee dome of total width 20 and height 17.32:
    
    rotate_extrude()ogee(10 * cos(60), 10 * sin(60));
    
 Formula: Elevation z is given by

    z = { 1-sqrt((2*t-t^2)/3) , t <  0.5 
        { sqrt((1-t^2)/3))    , t >= 0.5
    
 where t = 0 .. 1, returning z = 1 .. 0.

 */
    pts = [ for (t = [0 : 1 / steps : 1]) [ 
        t, 
        (t < 0.5)?(1-sqrt((2*t-pow(t,2))/3)):
                  (sqrt((1-pow(t,2))/3))
    ] ];
    scale([width, height])polygon(concat([ [0, 0] ], pts));
}
